#######################################################
# FSR with Cross-Loadings: Population Matrix Examples #
#######################################################

#Remove any workplace objects at the outset:
rm(list = ls())

#require the lavaan library
require("lavaan")

#############################
# Set Population Parameters #
#############################

#Names for objects
#factor model indicators:
wNames <- paste0("w", 1:4)
yNames <- paste0("y", 1:8)
indNames <- c(wNames, yNames)

#names of latent factors:
facNames <- paste0("eta", c("w", "1", "2"))
allNames <- c(indNames, facNames)

#set factor loadings
l <- 0.606977
lp <- 0.4875356
lc <- 0.2437678

#unique factor variances and covariances
varU <- 0.6315789
covU <- 0.3157895

lambda <- matrix(0, nrow = 12, ncol = 3, dimnames = list(indNames, facNames))

lambda[1:4,1] <- lambda[5:6, 2] <- lambda[11:12, 3] <- l

#numCrosses must be set to 1 or 2 to emulate the examples in the paper.
numCrosses <- 1

if(numCrosses == 1){
	lambda[7:8, 2] <- lp	
	lambda[7:8, 3] <- lc
	lambda[9:10, 3] <- l
}else if(numCrosses == 2){
	lambda[7:8, 2] <- lambda[9:10, 3] <- lp	
	lambda[7:8, 3] <- lambda[9:10, 2] <- lc
}else{
	stop("numCrosses must equal 1 or 2!")
}

theta <- diag(varU, nrow = 12, ncol = 12)
dimnames(theta) <- list(indNames, indNames)

#Change corU to TRUE to add correlated uniquenesses
corU <- FALSE
if(corU) theta["w1", "y8"] <- theta["y8", "w1"] <- theta["w2", "y7"] <- theta["y7", "w2"] <- theta["w3", "y2"] <- theta["y2", "w3"] <- theta["w4", "y1"] <- theta["y1", "w4"]  <- covU

phi <- matrix(c(1, sqrt(.3), sqrt(.3),
			   sqrt(.3), 1, .3,
			    sqrt(.3), .3, 1), nrow = 3, ncol = 3, dimnames = list(facNames, facNames))

#Covariance matrix of the observed variables
Sigma <- lambda%*%phi%*%t(lambda) + theta
diag(Sigma) <- 1 #smooth out the 0.9999999s

#############################################
# Model A and Model B Population Parameters #
#############################################

#set sample size
n <- 500

#Syntax for FSR models and/or true latent regressions
modASyn <- " eta1~ gamma1*etaw; eta2 ~ gamma2*etaw; eta1 ~~ dCov*eta2"
modBSyn <- "etaw ~ gamma1*eta1 + gamma2*eta2; eta1 ~~ cov12*eta2"

#Fit Structural Models to True Population Matrix
modAFitPop <- sem(modASyn, sample.cov = phi, sample.nobs = n, sample.cov.rescale = FALSE)
modBFitPop <- sem(modBSyn, sample.cov = phi, sample.nobs = n, sample.cov.rescale = FALSE)

#Assign true population estimates for later comparison
peModAPop <- parameterEstimates(modAFitPop, standardized = TRUE)
peModBPop <- parameterEstimates(modBFitPop, standardized = TRUE)

#Population Gammas (gamma1 == gamma2 in both models).
gammaPopModA <- peModAPop[peModAPop$label == "gamma1", "std.all"]
gammaPopModB <- peModBPop[peModBPop$label == "gamma1", "std.all"]

##############################################
# Bias in SEM Models Omitting Cross-Loadings #
##############################################

# SEM Models Assuming Simple Structure + Conditional Independence of All Uniquenesses

#Model A
modASynSS <- "
	etaw =~ w1 + w2 + w3 + w4
	eta1 =~ y1 + y2 + y3 + y4
	eta2 =~ y5 + y6 + y7 + y8
	eta1 ~ gamma1*etaw
	eta2 ~ gamma2*etaw
	eta1 ~~ dCov*eta2 #disturbance covariance
"

modAFitSS <- sem(modASynSS, sample.cov = Sigma, sample.nobs = n, sample.cov.rescale = FALSE)

#View results
summary(modAFitSS, standardized = TRUE)

#parameter estimates
pemodA_ss <- parameterEstimates(modAFitSS, standardized = TRUE)

#gammas
gammaModA_ss <- pemodA_ss[grep("gamma", pemodA_ss$label), "std.all"]

#Parameter Estimates in Population vs. Simple Structure SEM
gammaPopModA
gammaModA_ss

#Percent Bias
pbGammaModA_ss <- ((gammaModA_ss - gammaPopModA)/gammaPopModA)*100
#View percent bias
round(pbGammaModA_ss, 2)

#Significant disturbance covariance.
pemodA_ss[pemodA_ss$label == "dCov", c("label", "std.all", "pvalue")]

#Model B
modBSynSS <- "
	etaw =~ w1 + w2 + w3 + w4
	eta1 =~ y1 + y2 + y3 + y4
	eta2 =~ y5 + y6 + y7 + y8
	etaw ~ gamma1*eta1 + gamma2*eta2
	eta1 ~~ cov12*eta2 #exogenous covariance
"

modBFitSS <- sem(modBSynSS, sample.cov = Sigma, sample.nobs = n, sample.cov.rescale = FALSE)

#View results
summary(modBFitSS, standardized = TRUE)

#parameter estimates
pemodB_ss <- parameterEstimates(modBFitSS, standardized = TRUE)

#gammas for model B
gammaModB_ss <- pemodB_ss[grep("gamma", pemodB_ss$label), "std.all"]

#Parameter Estimates in Population vs. Simple Structure SEM
gammaPopModB
gammaModB_ss

#Percent Bias
pbGammaModB_ss <- ((gammaModB_ss - gammaPopModB)/gammaPopModB)*100
#View percent bias
round(pbGammaModB_ss, 2)


#********************************
# Exogenous and Disturbance Covs  
#********************************

semdCov <- pemodA_ss[pemodA_ss$label == "dCov", "std.all"]
semExoCov <- pemodB_ss[pemodB_ss$label == "cov12", "std.all"]

semdCov #should be zero
semExoCov #should be .3

###############
# FSR Methods #
###############

#========================
# Hoshino-Bentler Method
#========================

#Overall A matrix, Bartlett Estimation
AmatBart <- solve(t(lambda)%*%solve(theta)%*%lambda)%*%t(lambda)%*%solve(theta)

#Expected Variance-Covariance Matrix of the Bartlett Factor Scores
HBCovs <- AmatBart%*%Sigma%*%t(AmatBart)

#Hoshino-Bentler Correction to the Latent Factor Variances
diag(HBCovs) <- 1

#Note: HBCovs underestimate the true latent covariances in the connected measurement model.
HBCovs; phi

#Model A
HBFitModA <- sem(modASyn, sample.cov = HBCovs, sample.nobs = n, sample.cov.rescale = FALSE)

#Save parameter estimates.
peHB_ModA <- parameterEstimates(HBFitModA, standardized = TRUE)

#gammas
gammaModA_HB <- peHB_ModA[grep("gamma", peHB_ModA$label), "std.all"]

#Compare population estimates to HB estimates.
gammaPopModA
gammaModA_HB

#In this model, absent the correlated uniquenesses, the estimates are identical, because the covariances of etaw with eta1 and eta2 are intact.
pbGammaModA_HB <- ((gammaModA_HB - gammaPopModA)/gammaPopModA)*100 

#View percent bias
round(pbGammaModA_HB, 2) #no bias if corU = FALSE.

#However, the disturbance covariance is significant and negative (but should be zero in the true model):
peHB_ModA[peHB_ModA$label == "dCov", c("std.all", "pvalue")]

#Model B
HBFitModB <- sem(modBSyn, sample.cov = HBCovs, sample.nobs = n, sample.cov.rescale = FALSE)

#Save parameter estimates.
peHB_ModB <- parameterEstimates(HBFitModB, standardized = TRUE)

#gammas
gammaModB_HB <- peHB_ModB[grep("gamma", peHB_ModB$label), "std.all"]

#Compare population estimates to HB estimates.
gammaPopModB
gammaModB_HB

#In this model, the estimates are identical, because the covariances of etaw with eta1 and eta2 are intact.
pbGammaModB_HB <- ((gammaModB_HB - gammaPopModB)/gammaPopModB)*100 

#View percent bias
round(pbGammaModB_HB, 2)


#***************************
# Exo and Disturbance Covs  
#***************************

hbdCov <- peHB_ModA[peHB_ModA$label == "dCov", "std.all"]
hbExoCov <- peHB_ModB[peHB_ModB$label == "cov12", "std.all"]

hbdCov #should be zero
hbExoCov #should be .3

#=========================
# Skrondal & Laake Method
#=========================

#Subset matrices necessary for blockwise computations:
Sigmaw <- Sigma[wNames, wNames]
Sigmay <- Sigma[yNames, yNames]
Sigmayw <- Sigma[yNames, wNames]
Sigmawy <- Sigma[wNames, yNames]
lambdaw <- lambda[wNames, "etaw", drop = FALSE]
lambday <- lambda[yNames, c("eta1", "eta2")]
phiw <- phi["etaw", "etaw"]
phiy <- phi[c("eta1", "eta2"), c("eta1", "eta2")]
thetaw <- theta[wNames, wNames]
thetay <- theta[yNames, yNames]
thetawy <- theta[wNames, yNames]
thetayw <- theta[yNames, wNames]

#=========
# Model A
#=========

#Regression of etaw IV:
Amatw_reg <- phiw%*%t(lambdaw)%*%solve(Sigmaw)

Amaty_Bart <- solve(t(lambday)%*%solve(thetay)%*%lambday)%*%t(lambday)%*%solve(thetay)

varFSRegw <- Amatw_reg%*%Sigmaw%*%t(Amatw_reg)
covFSBartyFSRegw <- Amaty_Bart%*%Sigmayw%*%t(Amatw_reg)

#SL Gamma (see p. 569 of Skrondal & Laake, 2001 for formula):
gammaModA_SL <- covFSBartyFSRegw%*%solve(varFSRegw)

#Population versus SL:
gammaPopModA
gammaModA_SL

#percent bias
pbModA_SL <- ((gammaModA_SL - gammaPopModA)/gammaPopModA) * 100

#View percent bias
round(pbModA_SL, 2)

#=========
# Model B
#=========

Amaty_reg <- phiy%*%t(lambday)%*%solve(Sigmay)
Amatw_Bart <- solve(t(lambdaw)%*%solve(thetaw)%*%lambdaw)%*%t(lambdaw)%*%solve(thetaw)

varFSRegy <- Amaty_reg%*%Sigmay%*%t(Amaty_reg)
covFSBartwFSRegy <- Amatw_Bart%*%Sigmawy%*%t(Amaty_reg)

#SL Gamma (see p. 569 of Skrondal & Laake, 2001 for formula):
gammaModB_SL <- covFSBartwFSRegy%*%solve(varFSRegy)

#Population versus SL:
gammaPopModB
gammaModB_SL

#percent bias
pbModB_SL <- ((gammaModB_SL - gammaPopModB)/gammaPopModB) * 100
#View percent bias
round(pbModB_SL, 2)

#=======================================================================
# Croon's Method with Hayes-Usami Correction at Measurement Model Level
#=======================================================================

#For each model, we will demonstrate two approaches:
	#1. Applying the correction to the entire measurement model all at once.
	#2. Applying the measurement-model-by-measurement model approach.

#==========
# Model A
#==========

#Note: this approach will produce correct results in all cases.

#Approach 1:
BartlettCovs <- AmatBart%*%Sigma%*%t(AmatBart)

#Croon covs, using Approach 1
CroonCovsA1 <- BartlettCovs - AmatBart%*%theta%*%t(AmatBart)

#Identical to Phi Matrix:
CroonCovsA1; phi
all.equal(round(CroonCovsA1, 5), round(phi, 5)) #TRUE

#Approach 2: 

#This approach will produce identical results to the above
#in the absence of correlated uniquenesses in figure 2
#(when they are included, the entire three-factor measurement model is connected,
# and Approach 1 is required).

#Correct Covariances for etaw Factor Model:
BartlettCovsw <- Amatw_Bart%*%Sigmaw%*%t(Amatw_Bart)
CroonCovsw <- BartlettCovsw - Amatw_Bart%*%thetaw%*%t(Amatw_Bart)

#Correct Covariances for Connected eta1 and eta2 Measurement Model:
BartlettCovsy <- Amaty_Bart%*%Sigmay%*%t(Amaty_Bart)
CroonCovsy <- BartlettCovsy - Amaty_Bart%*%thetay%*%t(Amaty_Bart)

#Correct Covariances Across Models:
CroonCovswy <- Amatw_Bart%*%Sigmawy%*%t(Amaty_Bart) -Amatw_Bart%*%thetawy%*%t(Amaty_Bart)

#Full Croon Covariance Matrix using Model 2
CroonCovsA2 <- cbind(rbind(CroonCovsw, t(CroonCovswy)), rbind(CroonCovswy, CroonCovsy))

#Once again, this is identical to the population factor matrix, phi:
CroonCovsA2; phi

all.equal(round(CroonCovsA2, 5), round(phi, 5)) #TRUE in conditions with no unique factor covariances - that is, when corU == FALSE.

